# coding: utf-8
from django.forms.models import model_to_dict

from aio_client.provider.models import GetProviderReceipt
from aio_client.provider.models import GetProviderRequest
from aio_client.provider.models import PostProviderRequest

from .exceptions import ReceiptNotFound
from .exceptions import ResponseNotFound
from .helpers import provider_post_request


def get_requests(message_type=None):
    """Получение всех заявок к РИС
    :param message_type вид сведений, необязательный параметр,
    если не передается, отдаем все запрросы
    :return: список запросов к РИС как к поставщику услуг
    """
    qs = GetProviderRequest.objects.exclude(
        state=GetProviderRequest.SENT)
    if message_type:
        qs = qs.filter(message_type=message_type)
    result = list(qs.values(*GetProviderRequest.LIST_AIO_FIELDS))
    qs.update(state=GetProviderRequest.SENT)
    return result


def set_error_requests(origin_message_ids):
    """Выставляем признак ошибки при обработке сообщения,
    для возможности получить эти сообщения в get_requests повторно
    :param origin_message_ids: список origin_message_id
    :return: int кол-во измененных записей
    """
    assert type(origin_message_ids) is list
    qs = GetProviderRequest.objects.filter(
        origin_message_id__in=origin_message_ids)
    qs.update(state=GetProviderRequest.ERROR)
    return qs.count()


def push_request(message):
    """Передача ответа на на запрос услуги
    :param message запись aio_client.provider.models.PostProviderRequest
    :return: инстанс класса requests.models.Response
    """
    assert isinstance(message, PostProviderRequest)
    response = provider_post_request(message)
    return response


def get_response(origin_message_id):
    """Получение ответа от СМЭВ на ответ на запрос услуги
    :param origin_message_id str идентификаор сообщения
    :return dict со списком полей GetProviderReceipt.LIST_AIO_FIELDS
    """
    qs_response = PostProviderRequest.objects.filter(
        origin_message_id=origin_message_id)
    if not qs_response.exists():
        raise ResponseNotFound(
            message=ResponseNotFound.DEFAULT_MSG % origin_message_id)

    receipt = GetProviderReceipt.objects.filter(
        origin_message_id=origin_message_id).order_by('id').last()
    if not receipt:
        raise ReceiptNotFound(
            message=ReceiptNotFound.DEFAULT_MSG % origin_message_id)
    return model_to_dict(receipt, GetProviderReceipt.LIST_AIO_FIELDS)
