# coding: utf-8
import datetime

from celery import states
from celery.schedules import crontab

from aio_client import configs as aio_client_settings
from aio_client.base.exceptions import AioClientException
from aio_client.base.tasks import PeriodicAsyncTask
from aio_client.provider.helpers import provider_delete_receipt
from aio_client.provider.helpers import provider_delete_request
from aio_client.provider.helpers import provider_get_receipt
from aio_client.provider.helpers import provider_get_requests


class GetAllRequestsProvideTask(PeriodicAsyncTask):
    """Получает все заявки к РИС от СМЭВ и помещает их в БД.
    По каждой заявке отправляем запрос на удаление полученных заявок"""

    description = u"AIO клиент провайдер. Получение всех заявок к РИС."
    stop_executing = False
    LOG_TIME_FORMAT = "%d.%m.%Y %H:%M"
    abstract = not aio_client_settings.PROVIDER_ON
    if aio_client_settings.PROVIDER_ON:
        run_every = crontab(
            minute=aio_client_settings.PR_REQ_TASK_EVERY_MINUTE,
            hour=aio_client_settings.PR_REQ_TASK_EVERY_HOUR)
    else:
        run_every = None

    def run(self, *args, **kwargs):
        super(GetAllRequestsProvideTask, self).run(*args, **kwargs)
        values = {
            u"Время начала": datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT),
        }
        self.set_progress(
            progress=u"Получаем все заявки к РИС в качесве поставщика ...",
            values=values
        )

        messages = provider_get_requests()
        values[u'Кол-во сообщений'] = str(len(messages))
        for message in messages:
            message_id = message.get('message_id')
            values[u'Сообщение %s' % message_id] = (
                u'Отправка запроса на удаление в АИО')
            try:
                provider_delete_request(message_id)
            except AioClientException as exc:
                values[u'Сообщение %s на удаление не удалось отправить'
                       % message_id] = (
                    u'%s (код ошибки - %s)' % (exc.message, exc.code))
                continue
        values[u"Время окончания"] = datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT)

        self.set_progress(
            progress=u'Завершено',
            task_state=states.SUCCESS,
            values=values)

        return self.state


class GetAllReceiptsProvideTask(PeriodicAsyncTask):
    """Получение ответа СМЭВ по всем отправленным заявкам.
    По каждому ответу отправляем запрос на удаление полученных ответов от СМЭВ
    """

    description = (u"AIO клиент провайдер."
                   u" Получение ответа СМЭВ по всем отправленным заявкам.")
    stop_executing = False
    LOG_TIME_FORMAT = "%d.%m.%Y %H:%M"
    abstract = not aio_client_settings.PROVIDER_ON
    if aio_client_settings.PROVIDER_ON:
        run_every = crontab(
            minute=aio_client_settings.PR_REC_TASK_EVERY_MINUTE,
            hour=aio_client_settings.PR_REC_TASK_EVERY_HOUR)
    else:
        run_every = None

    def run(self, *args, **kwargs):
        super(GetAllReceiptsProvideTask, self).run(*args, **kwargs)
        values = {
            u"Время начала": datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT),
        }
        self.set_progress(
            progress=u"Получение ответа СМЭВ по всем отправленным заявкам ...",
            values=values
        )

        messages = provider_get_receipt()
        values[u'Кол-во квитанций'] = str(len(messages))
        if not aio_client_settings.DEBUG_MODE:
            for message in messages:
                message_id = message.get('message_id')
                values[u'Сообщение %s' % message_id] = (
                    u'Отправка запроса на удаление полученной квитанции '
                    u'от СМЭВ в АИО'
                )
                try:
                    provider_delete_receipt(message_id)
                except AioClientException as exc:
                    values[(u'Сообщение %s на удаление полученной квитанции '
                            u'от СМЭВ не удалось отправить' % message_id)] = (
                        u'%s (код ошибки - %s)' % (exc.message, exc.code))
                    continue
        values[u"Время окончания"] = datetime.datetime.now(
            ).strftime(self.LOG_TIME_FORMAT)
        self.set_progress(
            progress=u'Завершено',
            task_state=states.SUCCESS,
            values=values)

        return self.state
